# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996

####################################################
# Common routines to handle parameters             #
####################################################

# Translate the parametes from the short form to the longer form
# Pass in the short parm, then the list of translations 
#	example: f h help f parameter-file
#   where: f is the parameter to translate
#   and the translations are: h=help,f=parameter-file,etc
translate_parms () {
	short_parm=${1}
	LONG_PARM=
	shift
	while (( "$#" )); do
        # debug "translate $short_parm: $1"    		
        if [[ "$short_parm" == "$1" ]]; then
	   	    LONG_PARM=$2
#            debug "translation found $short_parm: $2"            
	        return 0
	    fi
	    shift
	done
	
	return 1	
}

# Check if the first parameter passed is a flag
# Pass in the parameter to check, then the list of valid flags 
#   example: f h help t terminate
#   where: f is the parameter to check
#   and the valid flags are: h help t terminate  etc
# Returns:
#   1 if the parameter is a flag
#   0 otherwise
#
parm_is_flag () {
    check_parm=${1}
    shift
    while (( "$#" )); do
#         debug "is flag: $check_parm against $1"          
        if [[ "$check_parm" == "$1" ]]; then
            return 1
        fi
        shift
    done
    
    return 0    
	
}

# See if this parameter is allowed to be specified multiple times
# There is a arbitrary maximum of 10 times a parameter can be specified. 
# Parameter
#  The parameter to check
#
# Returns the multi parm name via MULTI_PARM_VAR
#
check_multi_parm () {
	#reset our global return variable
    MULTI_PARM_VAR=
    
    local check_parm=${1}
    local i
    
    shift
    while (( "$#" )); do
    	# get the first candidate and translate '-' to '_', since variable can't contain a '-'
        local multi_parm=${1//-/_}
#        debug "check multi: $check_parm against $1"
                   
        if [[ "$check_parm" == "$multi_parm" ]]; then
            # Now we need to find the next variable name to use
            for i in {1..10}
            do
            	# try this name
                MULTI_PARM_VAR=${check_parm}__${i}
                # if we can dereference it and the length is zero then it's available to use
            	if [ -z "${!MULTI_PARM_VAR}" ]; then
            	   return 0
                fi 
            done
        fi
        shift
    done    
    return 0    
}

# Send a fatal error if a "single" parameter is specified with others
# Parameters
#   The parameter to check
#   The number of parameters total
# Relies on:
#   PARM_SINGLES="enable disable terminate help"
# Returns
#   0 if no error
#   1 if the parm is a dup
#
chk_single_parm () {

    local i
    
    # if there is only one parameter to check then there is no error
    if [ $2 -le 1 ]; then
        return 0
    fi
    
    # Otherwise, check if the parameter is a single
    chk_parm_in_list $1 $PARM_SINGLES
    return $?
}

# Check if the first parameter is in the list that follows
# Parameters
#   The parameter to check
#   A list of parameters
# Relies on:
#   nothing
# Returns
#   0 if not in the list
#   1 if the parm is in the list
#
chk_parm_in_list () {
	
    local chk_parm=${1}
    shift
    local parm_list=$*
    local i
    
    # Check if the parameter is in the list
    for i in $parm_list
    do
    	if [[ "${i}" == "${chk_parm}" ]]; then
    		# found it
    		return 1
    	fi
    done
    
    # Must not have found it
    return 0
}

# Return an error if a parameter is not valid on the command
# Parameters
#   The parameter to check
# Relies on:
#   PARM_VALID="enable disable terminate help"
# Returns
#   1 if no error
#   0 if the parm is not valid
#
chk_valid_parm () {
	
    local chk_parm=${1}
    
    if [[ "$PARM_VALID_CHECKS" == "off" ]]; then
        # pretend we found it
        return 1
    fi
    
    chk_parm_in_list ${chk_parm} $PARM_VALID
    
    return $?
}


#
# Parse the parameters and create variables for each
# Pass in the list of parameters to parse
#
# The list of parameter processed will be added to CMD_PARMS
# Duplicate parameters will be suffixed with two underscores and a number.  For instance duplicate__2
#
# Relies on these variables, example:
# PARM_FLAGS="h help t terminate"
# PARM_MULTI="network-alias"
# PARM_TRANSLATION="f parameter-file h help t terminate d disable e enable"
#
#
parse_parms () {
	
	# Save the number of parameters on the command line
	local parm_count=$#
	# while there are more parameters to parse
	while (( "$#" ));
	do
		orig_parm=${1}
	    parm=${1}
#       debug "Parm is $parm"
		# The parameter can be the long or short version
		# Check for the long version and parse out a value
		
		if [[ "${parm:0:2}" == "--" ]]; then
			# Get rid of --
			parm=${parm:2}
			# Delete everything up to the last = from the right
			option=${parm%%=*}

            # Check if this parameter is valid before we go any further
            chk_valid_parm $option
            if [ $? -eq 0 ]; then
                # TRANSLATORS: This error message means that a parameter $orig_parm was passed in to a script 
                # but this script does not recognize the parameter: 
                fatal_error "`eval_gettext \"Parameter not valid: \\\$orig_parm\"`"
            fi
			
			# if these are the same then there is no value set and let's valid it's just a flag
            if [[ "${parm}" == "${option}" ]]; then
                # See if this parameter is just a flag or if it should have a value
                parm_is_flag $option $PARM_FLAGS
                if [ $? -eq 0 ]; then
                    # TRANSLATORS: This error message means that a parameter $orig_parm was passed in to a script 
                    # without its required value.  For instance the user might run: 
                    # "smcli configureHA --secondary-node --password abcdefgh ..." but this is not valid because
                    # the configureHA command requires "--secondary-node" ($orig_parm) to have a value, like:
                    # smcli configureHA --secondary-node node2 --password abcdefgh
                	fatal_error "`eval_gettext \"Value required for option: \\\$orig_parm\"`"
                fi             
            	
                # It must be just a flag            	
                value=Y
            else
                parm_is_flag $option $PARM_FLAGS
                if [ $? -eq 1 ]; then
                    # TRANSLATORS: This error message means that a parameter $orig_parm was passed in to a script 
                    # with a value, but a value is not allowed.  For instance the user might run: 
                    # "smha --reset true" but this is not valid because the --reset ($orig_parm) parameter on the 
                    # smha command does not allow a value after the --reset flag, it should be called like:
                    # smcli --reset
                    fatal_error "`eval_gettext \"Value not allowed for option: \\\$orig_parm\"`"
                fi             
            
                # Get the value: delete everything up to the first = from the left
                value=${parm#*=}            
            fi            		 
            				
	    else 
	        # This is the short version of a parameter. Like -h
	        if [[ "${parm:0:1}" == "-" ]]; then
				parm=${parm:1}
				option=$parm
				
                # Check if this parameter is valid
                chk_valid_parm $option
                if [ $? -eq 0 ]; then
                    # TRANSLATORS: This error message means that a parameter $orig_parm was passed in to a script 
                    # but this script does not recognize the parameter: 
                    fatal_error "`eval_gettext \"Parameter not valid: \\\$orig_parm\"`"
                fi
				
				# See if this parameter is just a flag or if it should have a value
				parm_is_flag $option $PARM_FLAGS
                if [ $? -eq 1 ]; then
	                # This is a flag with no value but we'll make it Y so we can define it         
	                value=Y
	                
	                # Now check if they specified a value when there shouldn't be one
	                if [ -n "$2" ]; then
		                if [[ "${2:0:1}" != "-" ]]; then
                            # TRANSLATORS: This error message means that a parameter $orig_parm was passed in to a script 
                            # with a value, but a value is not allowed.  For instance the user might run: 
                            # "smha --reset true" but this is not valid because the --reset ($orig_parm) parameter on the 
                            # smha command does not allow a value after the --reset flag, it should be called like:
                            # smcli --reset
		                    fatal_error "`eval_gettext \"Value not allowed for option: \\\$orig_parm\"`"
		                fi
	                fi             
	                
	            else	
		            if [ -z "${2}" ]; then
                        # TRANSLATORS: This error message means that a parameter $orig_parm was passed in to a script 
                        # without its required value.  For instance the user might run: 
                        # "smcli configureHA --secondary-node --password abcdefgh ..." but this is not valid because
                        # the configureHA command requires "--secondary-node" ($orig_parm) to have a value, like:
                        # smcli configureHA --secondary-node node2 --password abcdefgh
		            	fatal_error "`eval_gettext \"Value required for option: \\\$orig_parm\"`"
		            fi
	                value=$2
	                # $2 isn't really a parameter so subtract it out
	                let "parm_count = parm_count - 1"
	                shift
	            fi
            else
                # This shouldn't happen.  Each parameter should start with -- or -
                fatal_error "`eval_gettext \"Invalid parameter syntax: \\\$orig_parm\"`"
	        fi
		fi

        #Translate the options from short to long names
        if [[ "${#option}" == "1"  ]]; then
            
            translate_parms $option $PARM_TRANSLATION
            if [ -z "$LONG_PARM" ]; then
                # TRANSLATORS: This error message means that a parameter $orig_parm was passed in to a script 
                # but this script does not recognize the parameter: 
                fatal_error "`eval_gettext \"Parameter not valid: \\\$orig_parm\"`"
            else
                option=$LONG_PARM
            fi
            
        fi

        if [[ "${option}" == "debug" ]]; then
            DEBUG=ON
        fi  

        # Check to see this parameter is not allowed with others
        chk_single_parm $option $parm_count
        if [ $? -eq 1 ]; then
        	fatal_error "`eval_gettext \"Parameter \\\$orig_parm not allowed with any other parameters\"`"
        fi
        			
#	    debug "option=value is: $option=$value"
        # Everything checks out...
	    # Create a variable with this value, but first subsitute "_" for "-"
	    # ...and account for mulit specified parms
        option=${option//-/_}
        MULTI_PARM_VAR=
        check_multi_parm $option $PARM_MULTI
        if [ -n "$MULTI_PARM_VAR" ]; then
        	# Use the multi value
        	option=$MULTI_PARM_VAR
        fi
        
        # Here's the tricky part... eval seems to work to create the new variable. 	    
	    eval ${option}=$value	    
	    
	    # Add to the list of parameters processed
	    CMD_PARMS="$CMD_PARMS $option"	    
	                                    
	    # Process the next parameter                	
		shift
	done
	
	debug "all parms: $CMD_PARMS"
}

# Pass in the file to process
#
# Parameters
#   The file to process
process_parm_file () {
    local file_name_parm=${1}
	if [ -e $file_name_parm ]; then
        file_parms=`awk '! /[\s]*#/ {print $0}' $file_name_parm`
#   	debug $file_parms
    	parse_parms $file_parms
    else
        # TRANSLATORS: This error message means the file specified in the $file_name_parm was 
        # not found on the local file system
        fatal_error "`eval_gettext \"File not found: \\\$file_name_parm\"`"
	fi
}

# Parse the parameters
# This function is called like: parse_all_parms $*
# Parameters
#   The list of parameters to parms.
parse_all_parms () {
	
	expand_single_parm_sequences $*
	#parse the parameters passed in (which should be from the command line)
	parse_parms $EXPANDED_PARMS
	
	#if a parameters file was specified process it.
    if [ ! -z "${parameter_file}" ]; then
        process_parm_file ${parameter_file}
    fi
 
}

expand_single_parm_sequences () {
	local parms=$*
	local num_parms=$#
	
	EXPANDED_PARMS=
	while (( "$#" ));
    do
    	local parm=$1
        if [[ "${parm:0:2}" == "--" ]]; then
            EXPANDED_PARMS="$EXPANDED_PARMS $parm"
        else
		    if [[ "${parm:0:1}" == "-" ]]; then
	    	    parm=${parm:1}
	    	    while (( "${#parm}" ));
	    	    do
	    	    	EXPANDED_PARMS="$EXPANDED_PARMS -${parm:0:1}"
	    	    	parm=${parm:1}
	    	    done
	        else
	            EXPANDED_PARMS="$EXPANDED_PARMS $parm"
	        fi
        fi
    	shift
    done
	# debug "EXPANDED_PARMS $EXPANDED_PARMS"
}


# Make sure these parameters were specified
# Parameters
#   The list of parameters to check.
# Relies on
#   CMD_PARMS
#   PARM_MULTI
# Fails with an error if the parameter wasn't specified
ensure_parms () {
	local i
	
	# For each parameter, check it
    for i in $*
    do
    	# Translate this parm to a variable
        option_var=${i//-/_}
        
        # check if this is a multi parm and if so just look for the first instance
        check_multi_parm $option_var $PARM_MULTI
        if [ -n "$MULTI_PARM_VAR" ]; then
        	# Just look for the first variable of multiple
        	option_var=${option_var}__1
        fi
        
        # See if this parm was specified    
        if [ -z "${!option_var}" ]; then
            # Not found
            local parm_long_name=--$i
            fatal_error "`eval_gettext \"The \\\$parm_long_name parameter must be specified with this form of command invocation.\"`"
        fi
        
    done
}

# Make sure one parm of two, and only one, is specified
# Parameters
#   Two parameters to check.
# Relies on
#   CMD_PARMS
#   PARM_MULTI
# Fails with an error if the parameter wasn't specified
ensure_parms_xor () {
	local i
    local parm_list=
    local cnt=0
	
	# For each parameter, check it
    for i in $1 $2
    do
    	# Translate this parm to a variable
        option_var=${i//-/_}
        
        # check if this is a multi parm and if so just look for the first instance
        check_multi_parm $option_var $PARM_MULTI
        if [ -n "$MULTI_PARM_VAR" ]; then
        	# Just look for the first variable of multiple
        	option_var=${option_var}__1
        fi
        
        parm_list="${parm_list} --$i"

        # See if this parm was specified    
        if [ -n "${!option_var}" ]; then
            # found
            (( cnt++ ))
        fi
        
    done

    if (( cnt != 1 )); then
        fatal_error "`eval_gettext \"One of these parameters must be specified with this form of command invocation: \\\$parm_list\"`"
    fi
}

# Make sure these parameters were not specified
# Parameters
#   The parameter to check against
#   The list of parameters to check.
# Relies on
#   CMD_PARMS
#   PARM_MULTI
# Fails with an error if the parameter was specified
ensure_parms_not () {
    local i
    # Record the controlling parm
    local not_parm=${1}
    local long_not_parm=--${1}
    # shift the parms over
    shift
    
    # For each parameter, check it
    for i in $*
    do
        # Translate this parm to a variable
        option_var=${i//-/_}
        
        # check if this is a multi parm and if so just look for the first instance
        check_multi_parm $option_var $PARM_MULTI
        if [ -n "$MULTI_PARM_VAR" ]; then
            # Just look for the first variable of multiple
            option_var=${option_var}__1
        fi
        
        # See if this parm was specified    
        if [ -n "${!option_var}" ]; then
            # Found
            local parm_long_name=--$i
            # See if the parameter is a flag or not and send a good message
            parm_is_flag $not_parm $PARM_FLAGS
            if [ $? -eq 1 ]; then
                # TRANSLATORS: This error message means that the two parameters that the user
                # specified for this command, $parm_long_name and ${long_not_parm} can't be
                # used together, only one of the parameters is allowed.
                fatal_error "`eval_gettext \"The \\\$parm_long_name parameter cannot be specified with \\\${long_not_parm}.\"`"
            else
                local not_parm_value=${!not_parm}
                # TRANSLATORS: This error message means that the two parameters that the user
                # specified for this command, $parm_long_name and ${long_not_parm} (which has a
                # value of ${not_parm_value} )can't be used together, only one of the parameters 
                # is allowed.
                fatal_error "`eval_gettext \"The \\\$parm_long_name parameter cannot be specified with \\\${long_not_parm}=\\\${not_parm_value}.\"`"
            fi
        fi
        
    done
}

# Check the key values
# Parameters
#   The parameter name
#   The list of valid key values
# Relies on
# Fails with an error if the parameter value is not valid
check_key_values () {
    local parmname=$1
    shift
    
    local i
    for i in $*
    do
        if [[ "${!parmname}" == "$i" ]]; then
            return
        fi
    done

    local parm_value=${!parmname}
    # TRANSLATORS: This error message means the value specified for the parameter
    # ${parmname} is not valid.  For instance, the --dhcp parameter (${parmname})
    # might only accept the values of "enabled" or "disabled", so if the user
    # specifies: "--dhcp=on", then this message would say, "Parameter value not
    # valid: --dhcp=on
    fatal_error_rc 22 "`eval_gettext \"Parameter value not valid: \\\${parmname}=\\\${parm_value}\"`"
}

# Print out all the parameters and their values
list_parm_vars () {
	local i
	
    for i in $CMD_PARMS
    do
    	if [[ "${i}" == "password" ]]; then
            debug "parm: ${i}=XXXXXXXXXXXX"
    	else
        	debug "parm: ${i}=${!i}"
        fi
    done	
}

# Construct a string that looks like the original parms
# Parameters
#   ORIG_PARMS - the list of orignal parms to process
#   EXCLUDE_PARMS - the list of any parms to exclude
#   FLAG_PARMS - the list of flags
# Output:
#   PARM_STRING
construct_parms_string () {

    local orig_parms=${ORIG_PARMS}
    local exclude_parms=${EXCLUDE_PARMS}
    local flags=${FLAG_PARMS}
    local parms_string=

    local parm
    # For each parameter, check it
    for parm in $orig_parms
    do
        # See if we should exculde this parm
        chk_parm_in_list $parm $exclude_parms
        if [ $? -eq 0 ]; then
            # Not in exclude so handle it.

            # Translate to non multi format
            local non_multi_var=${parm//__*/}

            # Now translate infor parameter format versus variable format
            local parm_string=${non_multi_var//_/-}

            parm_is_flag $parm_string $flags
            if [ $? -eq 0 ]; then
                parms_string="$parms_string --${parm_string}=${!parm}"
            else
                parms_string="$parms_string --${parm_string}"
            fi
        fi
    done

    PARM_STRING=$parms_string
}

