#!/bin/bash
# Licensed Materials - Property of IBM
#
# 09N6996
#
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/bin:/usr/bin:/usr/sbin/rsct/bin:/usr/sbin
PROGNAME=$(basename $0)
RUN_AS_ROOT=/opt/ibm/director/vmi/bin/execWithPrivilege
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv          # required by smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

# the following function writes the message to debug and to stderr
bkrt_debug () {
    debug "$*"
    #echo  "$PROGNAME: $*" 1>&2
}

is_drbd_full_mirroring_in_progress () {
	local mirror=$1
	local rc=0
    local mirror_state="UpToDate/Inconsistent"     # set if we are emulating resources
	
    if [ -z "$SMHA_EMULATED_RSRCS" ]
    then
        mirror_state=`drbd-overview | grep $mirror | awk '{print $4}'`
    fi
    debug "Mirror state for $mirror is:$mirror_state"
    case "$mirror_state" in
        UpToDate/Inconsistent)            
            rc=1
            ;;
        *)
            rc=0
            ;;
    esac
    return $rc
}

# the following function checks the status of HA initial mirroring
#
# Exit code: 0 - Initial mirroring is not in progress
#            1 - Initial mirroring is in progress
#            9 - Error trying to determine if mirroring is in progress

initialMirroringInProgress () {
    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_imp.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}

        is_drbd_full_mirroring_in_progress $mlab
        if [ $? -ne 0 ]; then
        	debug "$mlab is still mirroring initial data"
        	rm -f $tmpf  >/dev/null        	
        	return 1
        fi
                 
    done < $tmpf
        
    rm -f $tmpf  >/dev/null
    return 0
}

# the following function checks the status of HA
#
# Exit code: 0 - HA is up; This node is Active
#            1 - HA is stopped
#            2 - HA is not configured
#            3 - HA is "busy" (in a failover or transition state, or being configured or unconfigured)
#            4 - HA is currently running initial data mirroring
#            5 - HA is up; This node is Passive
#            9 - HA state is unknown

check_HA_status () {
    local stat_RC
    local RC=9      # assume unknown state
    local Status=Error
    local mirror_RC=

    $RUN_AS_ROOT smhapstatus.sh &> /dev/null
    stat_RC=$?

    bkrt_debug "smhapstatus returned $stat_RC"

    case $stat_RC in
        0)     initialMirroringInProgress
               mirror_RC=$?
               debug "mirror_RC=$mirror_RC"
               
               if [ $mirror_RC -eq 0 ]; then
                   # mirroring is not in progress
                   RC=0
                   Status=active
               else
                   # mirroring is in progress, so report a busy status due to mirroring
                   RC=4
                   Status=mirroring
               fi
               ;;
        3)     RC=5         # Not configured
               Status="passive" ;;
        4)     RC=2         # Not configured
               Status="not configured" ;;
        2|6|8) RC=3         # Terminating, Configuring or Pending (busy)
               Status=busy ;;
        7)     RC=1         # HA is stopped
               Status=stopped ;;
    esac

    bkrt_debug "HA status: $Status ($RC)"

    return $RC
}

# the following function is used to stop the resources under control of TSA, but
# the RSCT subsystem will not be stopped since it may be in use by other subsystems.
#
# Exit code: 0 - HA was stopped or must not be stopped
#            1 - HA could not be stopped

stop_HA () {

    if [ ! -f $BKRT_NO_HASTOP_FLAGFILE ]    # if permitted to stop HA during backup
    then
        bkrt_debug "Stop HA ($BKRT_NO_HASTOP_FLAGFILE is not present)"

        # save the current NominalState of the resource groups in the "flag file"
        # The saved NominalState values will be restored after the backup is complete
        # or, if there is some error in the backup processing, when the clean-up
        # is done.
        #
        # N.B. the saved NominalState values will also be in the backup, such that
        # when the backup is restored, on the first boot the smhamaint.sh script
        # will restore the NominalState values
 
        if save_rgs_state $BKRT_FLAGFILE
        then

            take_all_rg_offline

            sleep 15                   # give TSA some time to stop the resource groups
            wait_all_rg_offline
            return                     # return exit code from wait_all_rg_offine
        else
            bkrt_debug "Could not save NominalState values"
            rm -f $BKRT_FLAGFILE
            return 1
        fi
    fi

    return 0
}

# the following function is used to backup the TSA/RSCT data via the ctbackup command.
# The RSCT command must be used since it knowns how to backup the TSA/RSCT data in
# order to guarantee a consistent set of saved data.
#
# Exit code: 0 - success
#            1 - ctbackup reported a failure

call_ctbackup () {
    if ctbackup &> $BKRT_CTBACKUP_OUT
    then
        # check if there is "WARNING: failed consistency check for ...." in output
        if [ -s $BKRT_CTBACKUP_OUT ] && fgrep "WARNING: failed consistency check" $BKRT_CTBACKUP_OUT
        then
            :      # do nothing built-in command
        else
            bkrt_debug "ctbackup succeeded"
            rm -f $BKRT_CTBACKUP_OUT
            return 0
        fi
    fi

    bkrt_debug "ctbackup failed; see $BKRT_CTBACKUP_OUT"

    sm_msg "`eval_gettext "The system cannot prepare the high availability data for backup. Contact Software Support."`"

    return 1
}

# the following function removes any TSA/RSCT backup data.

remove_ctbackup () {
    if [ -d /var/ct.backup ]
    then
        rm -rf /var/ct.backup
    fi
}

# the following function disables the normal start of the RMC daemon at boot time.
# We can't call ctrestore when this script is called with "restore reboot", since
# SRC is not yet running. We will call ctrestore out of smhamaint.sh, but that
# script is not called until after normal RMC start. Therefore, we disable the
# normal start of RMC now; it will be started when ctrestore is called later.
#
# N.B. This "disable" is a bit of hack, since RSCT currenlty provides no
# mechanism to prevent the start of RMC. We will ask the RSCT team for an
# interface to do so in the future.

disable_ctrmc_start () {

    if [ -d /var/ct.backup ]
    then
        # only do the disable if a backup file is present
        bkrt_debug "disable RMC start"

        mv /etc/init.d/ctrmc /etc/init.d/ctrmc.HAsaved

        # the lines of the "here" document, including the label, contain leading
        # tab characters. DO NOT inadvertently remove while editing this file !!!!

        cat > /etc/init.d/ctrmc <<-EOF
			#!/bin/bash
			exit 0
		EOF

        chmod 550 /etc/init.d/ctrmc
    fi

    return 0
}

# the following function restarts HA after taking the backup. or after a failed backup.
# It nodes not return any errors, since at this point we don't want to fail the backup

restart_HA () {
    if [ -f $BKRT_FLAGFILE ]       # if we stopped HA
    then
        if ! restore_rgs_state $BKRT_FLAGFILE
        then
            bkrt_debug "Could not restore NominalState values"
        fi
        rm -f $BKRT_FLAGFILE
    fi

    return 0
}

# clean after any errors during backup

cleanup_bkrt () {

    if rw_lock write get CmdSMHA $PROGNAME 100000      # wait "forever"
    then
        rm -f ${HACMD_INPROGRESS}_bkrt               # allow calls to smha commands once write lock is freed
    fi

    restart_HA        # restart HA if it was stopped

    remove_ctbackup   # remove any ctbackup

    rw_lock write free CmdSMHA $PROGNAME

    return 0
}

BKRT_MODE=$1
BKRT_PHASE=$2
RC=0             # assume everything is ok 

if [ -z "$BKRT_MODE" -o -z "$BKRT_PHASE" ]
then
    bkrt_debug "missing parameters to HA backup/restore plugin"
    exit 1
fi

bkrt_debug "Called with $BKRT_MODE / $BKRT_PHASE"

case $BKRT_MODE in
    backup)
        case $BKRT_PHASE in
            validate)   # nothing to do
                ;;
            pre)

                if rw_lock write get CmdSMHA $PROGNAME 10
                then
                    SMHABKRT_ACQUIRED_LOCK=yes

                    if files_exist $HACMD_INPROGRESS ; then
                        stat_RC=3    # return "busy" error if another command is in progress
                    else
                        check_HA_status
                        stat_RC=$?
                    fi
                else
                    SMHABKRT_ACQUIRED_LOCK=no
                    stat_RC=3    # return "busy" error if cannot get lock
                fi

                case $stat_RC in 
                    0)                # This HA node is active
                      if stop_HA               # stop HA if permitted
                      then
                          call_ctbackup     # backup TSA/RSCT data
                          RC=$?
                          if [ $RC -eq 0 ]; then
                              touch ${HACMD_INPROGRESS}_bkrt               # lock out calls to certain HA commands
                          fi
                      else
                          sm_msg "`eval_gettext "The system cannot stop the high availability environment for backup. Contact Software Support."`"
                          RC=1
                      fi
                      ;;
                    1)                # HA is stopped
                      call_ctbackup
                      RC=$?
                      if [ $RC -eq 0 ]; then
                          touch ${HACMD_INPROGRESS}_bkrt               # lock out calls to certain HA commands
                      fi
                      ;;
                    2)                # HA is not configured; nothing to do
                      ;;
                    3)                # HA is busy
                      sm_msg "`eval_gettext "The high availability environment is busy and backup cannot be performed at this time. Wait a few minutes, then try again."`"
                      RC=1
                      ;;
                    4)                # HA is mirroring
                      sm_msg "`eval_gettext "The high availability environment is currently mirroring data from the active node to the passive node.  Backup cannot be performed at this time.  Use smhastatus -mr to monitor the status of the mirroring, then try again when all data is up to date."`"
                      RC=1
                      ;;
                    5)                # This HA node is passive
                      sm_msg "`eval_gettext "This node is currently the passive node in a high availability environment. Backup cannot be performed on the passive node.  Either run the backup from the active node, or use smcli failover to make this node the active node and then run the backup."`"
                      RC=1
                      ;;
                    *)                # HA error
                      sm_msg "`eval_gettext "The high availability environment encountered an error while preparing the backup. Contact Software Support."`"
                      RC=1
                      ;;
                esac

                if [ "$SMHABKRT_ACQUIRED_LOCK" = "yes" ]; then
                    rw_lock write free CmdSMHA $PROGNAME
                fi
                
                ;;
            post)
                cleanup_bkrt          # restart HA if it was stopped and remove ctbackup data
                ;;
            cleanup)
                cleanup_bkrt          # same action as for "post"
                ;;
            *)
                bkrt_debug "invalid phase parameter to HA backup plugin: $BKRT_PHASE"
                RC=1
                ;;
        esac
        ;;
    restore)
        case $BKRT_PHASE in
            validate)   # nothing to do
                ;;
            pre)        # nothing to do
                ;;
            reboot)
                disable_ctrmc_start       # prevent RMC from starting
                ;;
            cleanup)    # nothing to do
                ;;
            *)
                bkrt_debug "invalid phase parameter to HA restore plugin: $BKRT_PHASE"
                RC=1
                ;;
        esac
        ;;
    *)
        bkrt_debug "invalid mode parameter to HA backup/restore plugin: $BKRT_MODE"
        RC=1
        ;;
esac

bkrt_debug "Exiting script with $RC"

exit $RC
