#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END
 
# 
# 09N6996
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PARM_FLAGS="h help c check u upgrade"
PARM_MULTI=""
PARM_TRANSLATION="c check \
u upgrade \
h help" 
PARM_SINGLES="help check upgrade"
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help"


load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh


###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    # list_parm_vars
      
}


upgrade_model () { 
	# action should be either "check" or "upgrade"
	local action=${1}	
	local RC=0
	
	debug "upgrade_model entered: action=$action"
	# This script add a new target to the existing "depends on" relationships between the primary
	# DRBD resources and their respective DRBD "sync" equivalencies. This new target is the DRBD base
	# resource
	#


	# Get the ResoureHandle for the DRBD base (aggregate) resource
	DRBDB_RH=$(lsrsrc-api -s 'IBM.Application::Name == "drbdb" && ResourceType == 2::ResourceHandle' 2> /dev/null)
	RC=$?
	if [ $RC -ne 0 ]
	then
		debug "upgrade_model: failure on lsrsrsc-api to get the resource handle: $?"
  	  	return 2
	fi

	# Find each "primary depends on sync" relationship. The assumption is that the
	# relationship name has this pattern => drbdp-<mirrorName>-depon-drbds-<mirrorName>-equ

	export IFS='|'
	lsrsrc-api -D '|' -s "IBM.ManagedRelationship::Name LIKE 'drbdp-%-depon-drbds-%-equ'::Name::Target" 2> /dev/null | \
	
	#################
	# NOTE NOTE NOTE!  The following loop actually ends up executing as a sub-shell.  This means that you can't "return" a return code.
	# To work around that, we use ()'s to create another subshell, with the while statement following by an exit statement to return
	# the exit code back to the invoking shell.
	#################
	
	(
		local aRC=0

		while read Name RH
		do
			# strip the {}'s from the Target attribute value, which is an array of resource handles
			RH=${RH#\{}
			RH=${RH%\}}

	    	# Check if there is more than one RH in the list. If so, then we assume that the relationship has
	   		# already been udpated

	    	tRH=${RH#*,}

	    	if [ "$tRH" = "$RH" ]
	    	then
	    		debug "upgrade_model: tRH = RH - upgrade required."
	    		# Only one RH in the target.  What are we supposed to do about it?
	    		if [ "$action" = "upgrade" ]
	    		then 

	        		# Add the RH for the base DRBD resource to the target
	        		# and modify the relationship with the new value

	        		Target="{$DRBDB_RH,$RH}"    # new value has two RH

	        		result=$(LC_ALL=C chrsrc-api -s "IBM.ManagedRelationship::Name == '$Name'::Target::$Target" 2>&1)
	        		RC=$?
	        		if [ $RC -ne 0 ]
	        		then
	        			debug "upgrade_model: chrsrc-api failed to modify relationship: RC=$RC : error => $result"
						aRC=1
	        		else
	        			debug "upgrade_model: chrsrc successful"
	        		fi
	        	else 
	        		# action must be "check".  Return a 1 indicating that an upgrade is needed
	        		debug "upgrade_model: check specified. upgrade needed, setting aRC=1"
	        		aRC=1
	        	fi
	    	else
	       		debug "upgrade_model: no change"
	    	fi
			debug "upgrade_model: at loop end: aRC=$aRC"
		done
		exit $aRC
	)
	# Need to pick up the return code passed back via the sub-shell exit
	RC=$?
	debug "returning RC=$RC"
	return $RC
	
}

stop_model () {

	debug "stop_model: taking down resource group $DIRECTOR_RG"
	take_rg_offline_n_wait_nonfatal $DIRECTOR_RG
	debug "stop_model: taking down resource group $DB2_RG"
	take_rg_offline_n_wait_nonfatal $DB2_RG
	debug "stop_model: taking down resource group $STORAGE_RG"
	take_rg_offline_n_wait_nonfatal $STORAGE_RG
	debug "stop_model: taking down resource group $DRBD_RG"
	take_rg_offline_n_wait_nonfatal $DRBD_RG
	debug "stop_model: finished stopping resource groups"
}

start_model () {
	debug "start_model: starting resource group $DRBD_RG"
	start_n_wait_for_rg_online $DRBD_RG
	debug "start_model: starting resource group $STORAGE_RG"
	start_n_wait_for_rg_online $STORAGE_RG
	debug "start_model: starting resource group $DB2_RG"
	start_n_wait_for_rg_online $DB2_RG
	debug "start_model: starting resource group $DIRECTOR_RG"
	start_n_wait_for_rg_online $DIRECTOR_RG


	debug "start_model: Finished bringing resource groups online"
}

###################################
## Main code                     ##
###################################

if [ $# -eq 0 ]; then
    exit 0
fi

proc_parms $*


##
## Should only be invoked with either --check or --upgrade, NOT both.
## Run down through each and process them.
##

# If check is specified then perform the check and exit via the proc_check_model routine
if [ -n "$check" ]; then
	upgrade_model "check"
	RC=$?
fi

# If upgrade is specified then perform the processing to upgrade the TSA model and return directly from the 
# proc_upgrade_model routine
if [ -n "$upgrade" ]; then

	# stop that part of the TSA model involved in the upgrade
	stop_model

	upgrade_model "upgrade"
	RC=$?

	# restart the model
	start_model
fi

exit $RC

