#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or 
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
# 

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh

#
# The /opt/ibm/director/data/IPPreference.properties file won't exist if we are upgrading from
# GA1 to GA2.  Add it.  If it does exist (should not happen, just ensure robustness), ensure it is correct.
#
handle_IP_preferences_file () {
	if [ ! -e $IPPREFERENCE_PROPS ]; then
		# file doesn't exist.  Create it and set permissions/owner same as other property files. 
		/usr/bin/touch $IPPREFERENCE_PROPS
   		/bin/chmod 644 $IPPREFERENCE_PROPS
   		/bin/chown root:root $IPPREFERENCE_PROPS
   	fi
        # Now, ensure we have the floater addresses stored as a property in the file
        # for Director's use
        set_ip_preferences_list
}

# For GA2, the file /opt/ibm/director/lwi/conf/overrides/ha.properties gets
# created during an HA configuration.  This tells Director that for any 
# advanced manager activations, it should call our activation framework.
# Advanced managers didn't ship in the GA1 based appliance and this file does
# not exist on the node that is being upgraded.  Since HA is already configured
# we need to add it here, so that when director comes up, the activation
# framework will run for any advanced managers that ship as "activated".
handle_HA_property_file () 
{
   	
   	if [ ! -e $HA_PROPS ]; then
   		# file doesn't exist.  Create it. 
                /usr/bin/touch $HA_PROPS
   		/bin/chmod 600 $HA_PROPS
   		/bin/chown root:root $HA_PROPS
   	fi
        set_HA_enabled_for_activation
}


################## MAIN CODE #####################


# This script will be called by the "upgrade" framework as follows:
#  - This script, being shipped in the "deferred" directory, is called after the boot order has 
#    switched on the "way back up" during the reboot while processing an upgrade.  This means that 
#    it is safe to call new functions that may be coded in GA2 in the HA "include" directory, since 
#    at this point in the process, we are no longer worried about running scripts from "/spare". 
#  - It is ONLY called on the active node.  It is never called on the passive node.
#  - The framework ensures that the active node has DB2 successfully started before calling
#    the scripts in the "deferred" directory where this script lives
#  - The framework sorts the script names found in the "deferred" directory in alpha-numerical 
#    ascending order.  This script is therefore installed with a "40-" prefix on the name to ensure
#    that is runs AFTER the Director "migration restore" has been done.  This ensures that if
#    the migration save/restore copies/restores the files that we operate on, we are not overwritten
#    by the restore, but are instead working on the final copy produced by the migration.
#  - The fact that DB2 is started ensures that the DRBD mount points are also properly mounted
#    which ensures that we are working on the "mirrored" data copy for those files that are in 
#    mirrored storage.  All files that are modified by this script exist in mirrored storage, and 
#    changes that we make will be replicated by DRBD to the passive node.

# We only want to handle these upgrade considerations if HA is configured
does_ha_config_exist
if [ $? -eq 1 ]; then
    # Ensure the HA property file is created to support the activation framework.  This was
    # previously done right from the base CSDA upgrade_deferred script, but we are moving it to 
    # this HA owned script, since the framework is designed for this, and it is a much cleaner approach.
    handle_HA_property_file

    # Ensure the list of "preferred IP addresses" (i.e. the list of TSA managed floating addresses)
    # is encoded for Director's use
    handle_IP_preferences_file

    # Ensure that the IP address change notification is set to off.  The default value is on, we need to be
    # sure that it is set to off (GA1 to GA2 migration does not currently copy this configuration file so we 
    # need to turn it off again on an upgrade.
    turn_off_address_change_processing

    # We need to ensure that the Director non-stop daeomon is not configured.  Call common routine to shut it down.
    # This routine does the mirrored storage file changes.  The non-mirrored needs to be done somewhere
    # else since this is only run on the active node.
    deactivate_dir_server_nonstop_mirrored_files

    # Finally, we want to force a one-time address change processing to happen for all endpoints.  This will cause 
    # Director, on the initial startup, to make a list of all endpoints that need to be dealt with and to store that
    # list into persistent storage.  The plan is, that once all those endpoints have been dealt with, then Director
    # itself will turn off this property that says "run the address change processing once".
    force_address_change_processing_next_restart
fi

exit 0




